/*
   Copyright 2012-2022 Michael Pozhidaev <msp@luwrain.org>

   This file is part of LUWRAIN.

   LUWRAIN is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 3 of the License, or (at your option) any later version.

   LUWRAIN is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.
*/

package org.luwrain.app.mail;

import java.util.*;

import org.luwrain.core.*;
import org.luwrain.core.events.*;
import org.luwrain.core.queries.*;
import org.luwrain.controls.*;
import org.luwrain.controls.ListArea.*;
import org.luwrain.controls.ListUtils.*;
import org.luwrain.controls.reader.*;
import org.luwrain.pim.*;
import org.luwrain.pim.mail.*;
import org.luwrain.pim.mail.script.*;
import org.luwrain.app.base.*;

import static org.luwrain.script.ScriptUtils.*;
import static org.luwrain.core.DefaultEventResponse.*;

import static org.luwrain.app.mail.App.*;

final class MainLayout extends LayoutBase implements TreeListArea.LeafClickHandler<MailFolder>, ClickHandler<SummaryItem>
{
    final App app;
    final TreeListArea<MailFolder> foldersArea;
    final ListArea<SummaryItem> summaryArea;
    final ReaderArea messageArea;

    private final List<SummaryItem> summaryItems = new ArrayList<>();
    private MailFolder folder = null;
    private MailMessage message = null;

    MainLayout(App app)
    {
	super(app);
	this.app = app;
	final TreeListArea.Params<MailFolder> treeParams = new TreeListArea.Params<>();
        treeParams.context = getControlContext();
	treeParams.name = app.getStrings().foldersAreaName();
	treeParams.model = new FoldersModel(app);
	treeParams.leafClickHandler = this;
	this.foldersArea = new TreeListArea(treeParams) {
		@Override public boolean onSystemEvent(SystemEvent event)
		{
		    if (event.getType() == SystemEvent.Type.REGULAR)
			switch(event.getCode())
			{
			case PROPERTIES:
			    return onFolderProps();
			}
		    return super.onSystemEvent(event);
		}
	    };
	this.foldersArea.requery();
	this.summaryArea = new ListArea<>(listParams((params)->{
		    params.name = app.getStrings().summaryAreaName();
		    params.model = new ListModel<>(summaryItems);
		    params.clickHandler = this;
		    params.appearance = new DoubleLevelAppearance<SummaryItem>(getControlContext()){
			    @Override public boolean isSectionItem(SummaryItem item)
			    {
				return item.type == SummaryItem.Type.SECTION;
			    }
			};
		    params.transition = new DoubleLevelTransition<SummaryItem>(params.model){
			    @Override public boolean isSectionItem(SummaryItem item)
			    {
				return item.type == SummaryItem.Type.SECTION;
			    }
			};
		}));
	final ActionInfo
	fetchIncomingBkg = action("fetch-incoming-bkg", app.getStrings().actionFetchIncomingBkg(), new InputEvent(InputEvent.Special.F6), app::fetchIncomingBkg);
	setAreaLayout(AreaLayout.LEFT_RIGHT, foldersArea, actions(
								  action("remove-folder", app.getStrings().actionRemoveFolder(), new InputEvent(InputEvent.Special.DELETE), this::actRemoveFolder),
								  action("new-folder", app.getStrings().actionNewFolder(), new InputEvent(InputEvent.Special.INSERT), MainLayout.this::actNewFolder),
								  fetchIncomingBkg),
		      summaryArea, actions(
					   fetchIncomingBkg
					   ));
	messageArea = null;
    }

    @Override public boolean onLeafClick(TreeListArea<MailFolder> area, MailFolder folder)
    {
	this.folder = folder;
	this.summaryItems.clear();
	this.summaryItems.addAll(organizeSummary(app.getStoring().getMessages().loadNoDeleted(folder)));
	summaryArea.refresh();
	summaryArea.reset(false);
	setActiveArea(summaryArea);
	return true;
    }

    @Override public boolean onListClick(ListArea area, int index, SummaryItem item)
    {
	final MailMessage message = item.message;
	if (message == null)
	    return false;
	try {
	    if (message.getState() == MailMessage.State.NEW)
	    {
		message.setState(MailMessage.State.READ);
		summaryArea.refresh();
	    }
	    messageArea.setDocument(Utils.createDocForMessage(message, app.getStrings()), 128);
	    return true;
	}
	catch(PimException e)
	{
	    app.getLuwrain().crash(e);
	    return true;
	}
    }

    private boolean actNewFolder()
    {
	final MailFolder opened = foldersArea.opened();
	if (opened == null)
	    return false;
	final String name = app.getConv().newFolderName();
	if (name == null)
	    return true;
	final int selectedIndex = foldersArea.selectedIndex();
	final MailFolder newFolder = new MailFolder();
	newFolder.setTitle(name);
	app.getMailStoring().getFolders().save(opened, newFolder, Math.max(selectedIndex, 0));
	foldersArea.requery();
	foldersArea.refresh();
	return true;
    }

    private boolean actRemoveFolder()
    {
	final MailFolder opened = foldersArea.opened();
	if (opened == null)
	    return false;
	final int selectedIndex = foldersArea.selectedIndex();
	if (selectedIndex < 0)
	    return false;
	if (!app.getConv().removeFolder())
	    return true;
	app.getStoring().getFolders().remove(opened, selectedIndex);
	foldersArea.requery();
	foldersArea.refresh();
	return true;
    }

    private boolean onFolderProps()
    {
	final Object obj = foldersArea.selected();
	if (!(obj instanceof MailFolder))
	    return false;
	final FolderPropertiesLayout propsLayout = new FolderPropertiesLayout(app, (MailFolder)obj, ()->{
		app.layout(getAreaLayout());
		foldersArea.refresh();
		app.getLuwrain().announceActiveArea();
	    });
	app.layout(propsLayout.getLayout());
	app.getLuwrain().announceActiveArea();
	return true;
    }

    private boolean actDeleteMessage(ListArea summaryArea, boolean deleteForever)
    {
	/*
	  NullCheck.notNull(summaryArea, "summaryArea");
	final Object o = summaryArea.selected();
	if (o == null || !(o instanceof SummaryItem))
	    return false;
	final SummaryItem item = (SummaryItem)o;
	if (item.message == null)
	    return false;
	try {
	    if (deleteForever)
		base.storing.getMessages().delete(item.message); else
		item.message.setState(MailMessage.State.DELETED);
	}
	catch(PimException e)
	{
	    luwrain.crash(e);
	    return true;
	}
	*/
	return true;
    }

    private boolean actReply(ListArea summaryArea)
    {
	/*
	  NullCheck.notNull(base, "base");
	NullCheck.notNull(summaryArea, "summaryArea");
	final Object obj = summaryArea.selected();
	if (obj == null || !(obj instanceof SummaryItem))
	    return false;
	final SummaryItem summaryItem = (SummaryItem)obj;
	if (summaryItem.message == null)
	    return false;

	return base.hooks.makeReply(summaryItem.message);
	*/
	return true;
    }

    boolean saveAttachment(String fileName)
    {
	/*
	  if (currentMessage == null)
	    return false;
	File destFile = new File(luwrain.launchContext().userHomeDirAsFile(), fileName);
	destFile = Popups.file(luwrain, "Сохранение прикрепления", "Введите имя файла для сохранения прикрепления:", destFile, 0, 0);
	if (destFile == null)
	    return false;
	if (destFile.isDirectory())
	    destFile = new File(destFile, fileName);
	final org.luwrain.util.MailEssentialJavamail util = new org.luwrain.util.MailEssentialJavamail();
	try {
	    if (!util.saveAttachment(currentMessage.getRawMail(), fileName, destFile))
	    {
		luwrain.message("Целостность почтового сообщения нарушена, сохранение прикрепления невозможно", Luwrain.MESSAGE_ERROR);
		return false;
	    }
	}
	catch (Exception e)
	{
	    e.printStackTrace();
	    luwrain.message("Во время сохранения прикрепления произошла непредвиденная ошибка:" + e.getMessage());
	    return false;
	}
	luwrain.message("Файл " + destFile.getAbsolutePath() + " успешно сохранён", Luwrain.MESSAGE_OK);
	*/
	return true;
    }

    private String[] getCcExcludeAddrs()
    {
	/*
	final org.luwrain.core.Settings.PersonalInfo sett = org.luwrain.core.Settings.createPersonalInfo(luwrain.getRegistry());
	final String addr = sett.getDefaultMailAddress("");
	if (addr.trim().isEmpty())
	    return new String[0];
	return new String[]{addr};
	*/
	return null;
    }

    private List<SummaryItem> organizeSummary(MailMessage[] messages)
    {
	return null;
    }
}
